from typing import Optional
from fastapi import APIRouter, Depends, HTTPException, Query
from sqlalchemy.orm import Session

from app.db.session import get_db
from app.models.project import ProjectStatus
from app.schemas.project import (
    Project,
    ProjectCreate,
    ProjectUpdate,
    ProjectWithTodos,
    ProjectListResponse,
)
from app.schemas.todo import TodoListResponse
from app.crud import project as project_crud
from app.crud import todo as todo_crud

router = APIRouter()


@router.get("/", response_model=ProjectListResponse)
def get_projects(
    page: int = Query(1, ge=1, description="Page number"),
    per_page: int = Query(10, ge=1, le=100, description="Items per page"),
    status: Optional[ProjectStatus] = Query(None, description="Filter by status"),
    search: Optional[str] = Query(None, description="Search in name and description"),
    db: Session = Depends(get_db),
):
    """Get all projects with pagination and filtering."""
    skip = (page - 1) * per_page
    projects, total = project_crud.get_projects(
        db=db, skip=skip, limit=per_page, status=status, search=search
    )

    total_pages = (total + per_page - 1) // per_page
    has_next = page < total_pages
    has_prev = page > 1

    return ProjectListResponse(
        items=projects,
        total=total,
        page=page,
        per_page=per_page,
        has_next=has_next,
        has_prev=has_prev,
    )


@router.post("/", response_model=Project)
def create_project(project: ProjectCreate, db: Session = Depends(get_db)):
    """Create a new project."""
    return project_crud.create_project(db=db, project=project)


@router.get("/{project_id}", response_model=ProjectWithTodos)
def get_project(project_id: int, db: Session = Depends(get_db)):
    """Get a specific project with its todos."""
    db_project = project_crud.get_project(db=db, project_id=project_id)
    if db_project is None:
        raise HTTPException(status_code=404, detail="Project not found")
    return db_project


@router.put("/{project_id}", response_model=Project)
def update_project(
    project_id: int, project_update: ProjectUpdate, db: Session = Depends(get_db)
):
    """Update a project."""
    db_project = project_crud.update_project(
        db=db, project_id=project_id, project_update=project_update
    )
    if db_project is None:
        raise HTTPException(status_code=404, detail="Project not found")
    return db_project


@router.delete("/{project_id}")
def delete_project(project_id: int, db: Session = Depends(get_db)):
    """Delete a project."""
    success = project_crud.delete_project(db=db, project_id=project_id)
    if not success:
        raise HTTPException(status_code=404, detail="Project not found")
    return {"message": "Project deleted successfully"}


@router.get("/{project_id}/todos", response_model=TodoListResponse)
def get_project_todos(
    project_id: int,
    page: int = Query(1, ge=1, description="Page number"),
    per_page: int = Query(10, ge=1, le=100, description="Items per page"),
    db: Session = Depends(get_db),
):
    """Get all todos for a specific project."""
    # First verify the project exists
    db_project = project_crud.get_project(db=db, project_id=project_id)
    if db_project is None:
        raise HTTPException(status_code=404, detail="Project not found")

    # Get todos with pagination
    skip = (page - 1) * per_page
    todos, total = todo_crud.get_todos(
        db=db, skip=skip, limit=per_page, project_id=project_id
    )

    total_pages = (total + per_page - 1) // per_page
    has_next = page < total_pages
    has_prev = page > 1

    return TodoListResponse(
        items=todos,
        total=total,
        page=page,
        per_page=per_page,
        has_next=has_next,
        has_prev=has_prev,
    )


@router.put("/{project_id}/archive", response_model=Project)
def archive_project(project_id: int, db: Session = Depends(get_db)):
    """Archive a project."""
    db_project = project_crud.archive_project(db=db, project_id=project_id)
    if db_project is None:
        raise HTTPException(status_code=404, detail="Project not found")
    return db_project
