from typing import List
from fastapi import APIRouter, Depends, HTTPException, status
from sqlalchemy.orm import Session

from app.db.session import get_db
from app.crud import tag as tag_crud
from app.schemas.tag import Tag, TagCreate, TagUpdate, TagListResponse
from app.schemas.todo import Todo

router = APIRouter()


@router.get("/", response_model=TagListResponse)
def read_tags(skip: int = 0, limit: int = 100, db: Session = Depends(get_db)):
    """
    Retrieve tags with pagination.
    """
    tags = tag_crud.get_tags(db, skip=skip, limit=limit)
    total = tag_crud.get_tags_count(db)

    return TagListResponse(items=tags, total=total)


@router.post("/", response_model=Tag, status_code=status.HTTP_201_CREATED)
def create_tag(tag: TagCreate, db: Session = Depends(get_db)):
    """
    Create a new tag.
    """
    # Check if tag with this name already exists
    db_tag = tag_crud.get_tag_by_name(db, name=tag.name)
    if db_tag:
        raise HTTPException(status_code=400, detail="Tag with this name already exists")

    return tag_crud.create_tag(db=db, tag=tag)


@router.get("/{tag_id}", response_model=Tag)
def read_tag(tag_id: int, db: Session = Depends(get_db)):
    """
    Get a specific tag by ID.
    """
    db_tag = tag_crud.get_tag(db, tag_id=tag_id)
    if db_tag is None:
        raise HTTPException(status_code=404, detail="Tag not found")
    return db_tag


@router.put("/{tag_id}", response_model=Tag)
def update_tag(tag_id: int, tag: TagUpdate, db: Session = Depends(get_db)):
    """
    Update a tag.
    """
    # Check if tag exists
    db_tag = tag_crud.get_tag(db, tag_id=tag_id)
    if db_tag is None:
        raise HTTPException(status_code=404, detail="Tag not found")

    # Check if new name conflicts with existing tag
    if tag.name and tag.name != db_tag.name:
        existing_tag = tag_crud.get_tag_by_name(db, name=tag.name)
        if existing_tag:
            raise HTTPException(
                status_code=400, detail="Tag with this name already exists"
            )

    return tag_crud.update_tag(db=db, tag_id=tag_id, tag=tag)


@router.delete("/{tag_id}", status_code=status.HTTP_204_NO_CONTENT)
def delete_tag(tag_id: int, db: Session = Depends(get_db)):
    """
    Delete a tag.
    """
    success = tag_crud.delete_tag(db, tag_id=tag_id)
    if not success:
        raise HTTPException(status_code=404, detail="Tag not found")


@router.post("/{tag_id}/todos/{todo_id}", status_code=status.HTTP_201_CREATED)
def add_tag_to_todo(tag_id: int, todo_id: int, db: Session = Depends(get_db)):
    """
    Add a tag to a todo.
    """
    success = tag_crud.add_tag_to_todo(db, todo_id=todo_id, tag_id=tag_id)
    if not success:
        raise HTTPException(
            status_code=400,
            detail="Failed to add tag to todo. Tag or todo not found, or tag already assigned.",
        )
    return {"message": "Tag successfully added to todo"}


@router.delete("/{tag_id}/todos/{todo_id}", status_code=status.HTTP_204_NO_CONTENT)
def remove_tag_from_todo(tag_id: int, todo_id: int, db: Session = Depends(get_db)):
    """
    Remove a tag from a todo.
    """
    success = tag_crud.remove_tag_from_todo(db, todo_id=todo_id, tag_id=tag_id)
    if not success:
        raise HTTPException(
            status_code=400,
            detail="Failed to remove tag from todo. Tag or todo not found, or tag not assigned.",
        )


@router.get("/{tag_id}/todos", response_model=List[Todo])
def get_todos_by_tag(
    tag_id: int, skip: int = 0, limit: int = 100, db: Session = Depends(get_db)
):
    """
    Get all todos that have a specific tag.
    """
    # Check if tag exists
    db_tag = tag_crud.get_tag(db, tag_id=tag_id)
    if db_tag is None:
        raise HTTPException(status_code=404, detail="Tag not found")

    return tag_crud.get_todos_by_tag(db, tag_id=tag_id, skip=skip, limit=limit)
